<?php

namespace App\Controller\Project;
/**
 * Class Parser
 * @package App\Controller\Project
 * @author Artsiom Malkovich (skype: artmalk)
 *
 */
class Parser extends \App\Controller\Admin
{

    public function  startBackgroundConnection($headers = array())
    {
       // return $this;
        ignore_user_abort(1);
        set_time_limit(0);
        session_write_close();
        header("Connection: close");
        ob_start();
        foreach ($headers as $header) {
            header($header);
        }
        $size = ob_get_length();
        header("Content-Length: $size");
        ob_end_flush();
        flush();

        return $this;
    }

    public function indexAction()
    {
        $project = $this->getProject(1);
        if (!$project->id) {
            $this->redirect(PROJECT_FOLDER);
        }


        if ($this->request()->isPost()) {
            $this->startBackgroundConnection()->go($project);exit;
        }

        $this->render('project/parser/index', array(
            'layout' => 'layouts/project.php',
            'project' => $project
        ));

    }

    protected function go($project)
    {
        $parser = $project->getLastParser();
        if ($parser) {
            if ($parser->getStatus() === \App\Model\Parser::STATUS_WORKING) {
                echo 'parser already running.';
                exit;
            }
        }

        if (!$parser || $parser->getStatus() === \App\Model\Parser::STATUS_FINISH) {
            $parser = \R::dispense('parser');
            $parser->projects_id = 1;
            $parser->links_on_start = count($project->ownBacklinks);
            $notices = $project->ownNotices;
            \R::trashAll($notices);
        }
        $parser->error = '';
        $parser->update = time();
        \R::store($parser);

        $backlinks = \R::find('backlinks', 'projects_id = 1');
        foreach ($backlinks as $backlink) {
            if ($backlink->parser_id === $parser->id) {
                continue;
            }
            try {
                $this->parseBacklink($parser, $backlink);
                unset ($backlink);//clean memory
            } catch (\Exception $e) {
                $parser->update = 0; //parser to fail
                $parser->error = $e->getMessage()."\n".$e->getTraceAsString();
                break;
            }
        }
        if (!$parser->error) {
            $parser->finished = \R::$f->now();
            $parser->sendNotices();
        }
        \R::store($parser);
    }

    protected function parseBacklink($parser, $backlink)
    {
        $backlink->parse();
        $backlink->parser_id = $parser->id;
        \R::store($backlink);

        $parser->dofollow += $backlink->getDoFollow();
        $parser->nofollow += $backlink->getNoFollow();
        $parser->links += $backlink->getFoundedLinks();
        $parser->update = time();
        $parser->updated_backlinks++;
        \R::store($parser);

    }

    /**
     * Add new links to statistic of last parser
     */
    public function updateAction($id)
    {
        $project = $this->getProject($id);
        if (!$project->id) {
            $this->redirect(PROJECT_FOLDER);
        }
        if ($this->request()->isPost()) {
            $lastParser = $project->getlastParser();
            if ($lastParser && $lastParser->getStatus() === \App\Model\Parser::STATUS_FINISH &&
                $lastParser->getParsedCount() < count($project->ownBacklinks) && $lastParser->can_update
            ) {
                $lastParser->finished = NULL;
                $lastParser->update = 0; //parser to fail
                $lastParser->links_on_start = count($project->ownBacklinks);
                $lastParser->error = '';
                \R::store($lastParser);
                $this->startBackgroundConnection(array('Location: '.PROJECT_FOLDER.'project/1/parser'))->go($project);
                exit;
            }
        }

        $this->redirect(PROJECT_FOLDER.'project/backlinks/1');

    }

    public function linkAction($id)
    {

        if (is_numeric($id)) {
            $backlink = \R::load('backlinks', $id);
        }

        if (!$backlink->id || !$this->request()->isPost()) {
            $this->redirect(PROJECT_FOLDER);
        }

        /** @var  $project \App\Model\Project */
        $project = $backlink->projects;
        $lastParser = $project->getLastParser();

        if ($lastParser && $lastParser->getStatus() === \App\Model\Parser::STATUS_FINISH && $lastParser->can_update) {
            //set parser status failed to continue
            $lastParser->finished = NULL;
            $lastParser->update = time();
            if ($backlink->parser_id !== $lastParser->id) {
                $lastParser->links_on_start++; //it is a new link
            } else {
                //old backlink. reset parser statistic about it
                $lastParser->links -= $backlink->getFoundedLinks();
                $lastParser->dofollow -= $backlink->getDoFollow();
                $lastParser->nofollow -= $backlink->getNoFollow();
                $lastParser->updated_backlinks--;
                $backlink->parser_id = 0;
                \R::store($backlink);
            }
            \R::store($lastParser);
            try {
                $this->startBackgroundConnection()->parseBacklink($lastParser, $backlink);
                $lastParser->finished = \R::$f->now();
            } catch (\Exception $e) {
                $lastParser->error = $e->getMessage()."\n".$e->getTraceAsString();
                $lastParser->update = 0;
            }
            \R::store($lastParser);

        }

        $this->redirect(PROJECT_FOLDER.'project/backlinks/1');


    }

    public function cronAction()
    {
        if (!defined('CRON')) {
            $this->redirect(PROJECT_FOLDER);
        }
        if (count(\R::findAll('parser', 'finished IS NULL')) > PARSERS_FORK_COUNT) {
            echo 'Only ' . PARSERS_FORK_COUNT . ' can be running one time by cron.';
            exit;
        }

        $projects = \R::findAll('projects', ' ORDER BY created');

        /* @var $project \App\Model\Project */
        foreach ($projects as $project) {

            $lastParser = $project->getLastParser();
            if ($lastParser) {
                if ($lastParser->getStatus() == \App\Model\Parser::STATUS_PAUSE_FAIL) {
                    $this->go($project);
                    exit; //only one parser for cron
                } elseif ($lastParser->getStatus() == \App\Model\Parser::STATUS_FINISH && $project->enable_cron) {
                    if ($project->cron_next_run <= time()) {
                        $nextRun = (\Cron\CronExpression::factory($project->cron)->getNextRunDate()->format('U'));
                        if ($nextRun - $project->cron_next_run < 1/*23 * 3600*/) { //no need run less than every 24-1(for lost)hrs
                            $nextRun = (\Cron\CronExpression::factory($project->cron)->getNextRunDate(null, 1)->format('U'));
                        }
                        $project->cron_next_run = $nextRun;
                        \R::store($project);
                        $this->go($project);
                        exit; //only one parser for cron
                    }
                }
            }

        }
        exit;
    }
}